@@ -32,6 +32,7 @@
 #include <string.h>
 #include <netinet/in.h>
 
+
 void send_row_description(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 						  short num_fields, char **field_names)
 {
@@ -100,13 +101,22 @@ void send_row_description(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backe
 	pool_flush(frontend);
 }
 
-void send_complete_and_ready(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, const int num_rows)
+/*
+ * send the command complete and ready for query message
+ * to frontend.
+ * if the num_row is passed with a -ve value it is not included
+ * to the command complete message
+ */
+void send_complete_and_ready(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,const char* message ,const int num_rows)
 {
 	int len;
 	int msg_len;
 	char msg[16];
 
-	msg_len = snprintf(msg, 16, "SELECT %d", num_rows);
+	if (num_rows >= 0)
+		msg_len = snprintf(msg, 16, "%s %d", message,num_rows);
+	else
+		msg_len = snprintf(msg, 16, "%s", message);
 
 	/* if we had more than 16 bytes, including '\0', the string was truncatured
 	 * shouldn't happen though, as it would means more than "SELECT 99999999" */
@@ -229,16 +239,6 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	StrNCpy(status[i].desc, "directory containing CA root certificate(s)", POOLCONFIG_MAXDESCLEN);
 	i++;
 
-	StrNCpy(status[i].name, "ssl_ciphers", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->ssl_ciphers);
-	StrNCpy(status[i].desc, "allowed SSL ciphers", POOLCONFIG_MAXDESCLEN);
-	i++;
-
-	StrNCpy(status[i].name, "ssl_prefer_server_ciphers", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->ssl_prefer_server_ciphers);
-	StrNCpy(status[i].desc, "Use server's SSL cipher preferences", POOLCONFIG_MAXDESCLEN);
-	i++;
-
 	/* POOLS */
 
 	/* - Pool size -  */
@@ -270,7 +270,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 
 	StrNCpy(status[i].name, "child_max_connections", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->child_max_connections);
-	StrNCpy(status[i].desc, "if max_connections received, child exits", POOLCONFIG_MAXDESCLEN);
+	StrNCpy(status[i].desc, "if max_connections received, chile exits", POOLCONFIG_MAXDESCLEN);
 	i++;
 
 	StrNCpy(status[i].name, "connection_life_time", POOLCONFIG_MAXNAMELEN);
@@ -287,7 +287,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 
 	/* - Where to log - */
 	StrNCpy(status[i].name, "log_destination", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->log_destination);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->log_destination_str);
 	StrNCpy(status[i].desc, "logging destination", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -333,7 +333,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	i++;
 
 	StrNCpy(status[i].name, "log_standby_delay", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->log_standby_delay);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->log_standby_delay);
 	StrNCpy(status[i].desc, "how to log standby delay", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -348,12 +348,6 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	StrNCpy(status[i].desc, "syslog program ident string", POOLCONFIG_MAXDESCLEN);
 	i++;
 
-	/* - Debug - */
-	StrNCpy(status[i].name, "debug_level", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->debug_level);
-	StrNCpy(status[i].desc, "debug message level", POOLCONFIG_MAXDESCLEN);
-	i++;
-
 	/* FILE LOCATIONS */
 
 	StrNCpy(status[i].name, "pid_file_name", POOLCONFIG_MAXNAMELEN);
@@ -465,7 +459,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	i++;
 
 	StrNCpy(status[i].name, "master_slave_sub_mode", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->master_slave_sub_mode);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->master_slave_sub_mode);
 	StrNCpy(status[i].desc, "master/slave sub mode", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -491,7 +485,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	i++;
 
 	StrNCpy(status[i].name, "delay_threshold", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%lld", pool_config->delay_threshold);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, INT64_FORMAT, pool_config->delay_threshold);
 	StrNCpy(status[i].desc, "standby delay threshold", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -643,7 +637,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	i++;
 
 	StrNCpy(status[i].name, "wd_lifecheck_method", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->wd_lifecheck_method);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->wd_lifecheck_method);
 	StrNCpy(status[i].desc, "method of watchdog lifecheck", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -785,7 +779,7 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	i++;
 
 	StrNCpy(status[i].name, "memqcache_method", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->memqcache_method);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->memqcache_method);
 	StrNCpy(status[i].desc, "Cache store method. either shmem(shared memory) or Memcached. shmem by default", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -903,12 +897,12 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 		i++;
 
 		snprintf(status[i].name, POOLCONFIG_MAXNAMELEN, "backend_status%d", j);
-		snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", BACKEND_INFO(j).backend_status);
+		snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s",backend_status_to_str(BACKEND_INFO(j).backend_status) );
 		snprintf(status[i].desc, POOLCONFIG_MAXDESCLEN, "status of backend #%d", j);
 		i++;
 
 		snprintf(status[i].name, POOLCONFIG_MAXNAMELEN, "standby_delay%d", j);
-		snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%lld", BACKEND_INFO(j).standby_delay);
+		snprintf(status[i].value, POOLCONFIG_MAXVALLEN, UINT64_FORMAT, BACKEND_INFO(j).standby_delay);
 		snprintf(status[i].desc, POOLCONFIG_MAXDESCLEN, "standby delay of backend #%d", j);
 		i++;
 
@@ -962,6 +956,109 @@ POOL_REPORT_CONFIG* get_config(int *nrows)
 	return status;
 }
 
+void send_config_var_detail_row(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, const char* name, const char* value, const char* description)
+{
+	int size;
+	int hsize;
+	static short num_fields = 3;
+
+	if (MAJOR(backend) == PROTO_MAJOR_V2)
+	{
+
+		int nbytes = (num_fields + 7)/8;
+		static unsigned char nullmap[2] = {0xff, 0xff};
+
+		/* ascii row */
+		pool_write(frontend, "D", 1);
+		pool_write_and_flush(frontend, nullmap, nbytes);
+		
+		size = strlen(name);
+		hsize = htonl(size+4);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)name, size);
+		
+		size = strlen(value);
+		hsize = htonl(size+4);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)value, size);
+		
+		size = strlen(description);
+		hsize = htonl(size+4);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)description, size);
+	}
+	else
+	{
+		short s;
+
+		pool_write(frontend, "D", 1);
+		size = 6; /* int32 + int16; */
+		size += 4 + strlen(name);  /* int32 + data; */
+		size += 4 + strlen(value); /* int32 + data; */
+		size += 4 + strlen(description);  /* int32 + data; */
+		hsize = htonl(size);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		s = htons(num_fields);
+		pool_write(frontend, &s, sizeof(s));
+		
+		size = strlen(name);
+		hsize = htonl(size);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)name, size);
+
+		size = strlen(value);
+		hsize = htonl(size);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)value, size);
+
+		size = strlen(description);
+		hsize = htonl(size);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)description, size);
+	}
+}
+
+void send_config_var_value_only_row(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, const char* value)
+{
+	int size;
+	int hsize;
+	static short num_fields = 1;
+	
+	if (MAJOR(backend) == PROTO_MAJOR_V2)
+	{
+		
+		int nbytes = (num_fields + 7)/8;
+		static unsigned char nullmap[2] = {0xff, 0xff};
+
+		/* ascii row */
+		pool_write(frontend, "D", 1);
+		pool_write_and_flush(frontend, nullmap, nbytes);
+		
+		size = strlen(value);
+		hsize = htonl(size+4);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)value, size);
+
+	}
+	else
+	{
+		short s;
+		
+		pool_write(frontend, "D", 1);
+		size = 6; /* int32 + int16; */
+		size += 4 + strlen(value); /* int32 + data; */
+		hsize = htonl(size);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		s = htons(num_fields);
+		pool_write(frontend, &s, sizeof(s));
+		
+		size = strlen(value);
+		hsize = htonl(size);
+		pool_write(frontend, &hsize, sizeof(hsize));
+		pool_write(frontend, (void*)value, size);
+	}
+}
+
 void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 {
 	static char *field_names[] = {"item", "value", "description"};
@@ -1032,7 +1129,7 @@ void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 		}
 	}
 
-	send_complete_and_ready(frontend, backend, nrows);
+	send_complete_and_ready(frontend, backend,"SELECT", nrows);
 
 	pfree(status);
 }
@@ -1042,6 +1139,7 @@ POOL_REPORT_NODES* get_nodes(int *nrows)
 	int i;
 	POOL_REPORT_NODES* nodes = palloc(NUM_BACKENDS * sizeof(POOL_REPORT_NODES));
 	BackendInfo *bi = NULL;
+	POOL_SESSION_CONTEXT *session_context = pool_get_session_context(false);
 
     for (i = 0; i < NUM_BACKENDS; i++)
 	{
@@ -1050,15 +1148,26 @@ POOL_REPORT_NODES* get_nodes(int *nrows)
 		snprintf(nodes[i].node_id, 	POOLCONFIG_MAXIDLEN, 	"%d", 	i);
 	    StrNCpy(nodes[i].hostname, 	bi->backend_hostname, 		strlen(bi->backend_hostname)+1);
 	    snprintf(nodes[i].port, 	POOLCONFIG_MAXPORTLEN, "%d", 	bi->backend_port);
-	    snprintf(nodes[i].status, 	POOLCONFIG_MAXSTATLEN, 	"%d", 	bi->backend_status);
+	    snprintf(nodes[i].status, 	POOLCONFIG_MAXSTATLEN, 	"%s", 	backend_status_to_str(bi->backend_status));
 	    snprintf(nodes[i].lb_weight, POOLCONFIG_MAXWEIGHTLEN, "%f", bi->backend_weight/RAND_MAX);
-	    snprintf(nodes[i].select, POOLCONFIG_MAXWEIGHTLEN, "%lld", stat_get_select_count(i));
+	    snprintf(nodes[i].select, POOLCONFIG_MAXWEIGHTLEN, UINT64_FORMAT, stat_get_select_count(i));
+	    snprintf(nodes[i].load_balance_node, POOLCONFIG_MAXWEIGHTLEN, "%s",
+				 (session_context->load_balance_node_id == i)? "true":"false");
 
-		if (MASTER_SLAVE && !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP))
+		snprintf(nodes[i].delay, POOLCONFIG_MAXWEIGHTLEN, "%d", 0);
+
+		if (STREAM)
+		{
 			if (i == REAL_PRIMARY_NODE_ID)
+			{
 				snprintf(nodes[i].role, POOLCONFIG_MAXWEIGHTLEN, "%s", "primary");
+			}
 			else
+			{
 				snprintf(nodes[i].role, POOLCONFIG_MAXWEIGHTLEN, "%s", "standby");
+				snprintf(nodes[i].delay, POOLCONFIG_MAXWEIGHTLEN, UINT64_FORMAT, bi->standby_delay);
+			}
+		}
 		else
 		{
 			if (i == REAL_MASTER_NODE_ID)
@@ -1075,7 +1184,7 @@ POOL_REPORT_NODES* get_nodes(int *nrows)
 
 void nodes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 {
-	static char *field_names[] = {"node_id","hostname", "port", "status", "lb_weight", "role", "select_cnt"};
+	static char *field_names[] = {"node_id","hostname", "port", "status", "lb_weight", "role", "select_cnt", "load_balance_node", "replication_delay"};
 	short num_fields = sizeof(field_names)/sizeof(char *);
 	int i;
 	short s;
@@ -1132,6 +1241,16 @@ void nodes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 			hsize = htonl(size+4);
 			pool_write(frontend, &hsize, sizeof(hsize));
 			pool_write(frontend, nodes[i].select, size);
+
+			size = strlen(nodes[i].load_balance_node);
+			hsize = htonl(size+4);
+			pool_write(frontend, &hsize, sizeof(hsize));
+			pool_write(frontend, nodes[i].load_balance_node, size);
+
+			size = strlen(nodes[i].delay);
+			hsize = htonl(size+4);
+			pool_write(frontend, &hsize, sizeof(hsize));
+			pool_write(frontend, nodes[i].delay, size);
 		}
 	}
 	else
@@ -1148,6 +1267,8 @@ void nodes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 			len += 4 + strlen(nodes[i].lb_weight); /* int32 + data; */
 			len += 4 + strlen(nodes[i].role);      /* int32 + data; */
 			len += 4 + strlen(nodes[i].select);    /* int32 + data; */
+			len += 4 + strlen(nodes[i].load_balance_node);    /* int32 + data; */
+			len += 4 + strlen(nodes[i].delay);    /* int32 + data; */
 			len = htonl(len);
 			pool_write(frontend, &len, sizeof(len));
 			s = htons(num_fields);
@@ -1180,10 +1301,18 @@ void nodes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 			len = htonl(strlen(nodes[i].select));
 			pool_write(frontend, &len, sizeof(len));
 			pool_write(frontend, nodes[i].select, strlen(nodes[i].select));
+
+			len = htonl(strlen(nodes[i].load_balance_node));
+			pool_write(frontend, &len, sizeof(len));
+			pool_write(frontend, nodes[i].load_balance_node, strlen(nodes[i].load_balance_node));
+
+			len = htonl(strlen(nodes[i].delay));
+			pool_write(frontend, &len, sizeof(len));
+			pool_write(frontend, nodes[i].delay, strlen(nodes[i].delay));
 		}
 	}
 
-	send_complete_and_ready(frontend, backend, nrows);
+	send_complete_and_ready(frontend, backend, "SELECT", nrows);
 
 	pfree(nodes);
 }
@@ -1384,7 +1513,7 @@ void pools_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 		pool_write(frontend, connected, len);
 	}
 
-	send_complete_and_ready(frontend, backend, nrows);
+	send_complete_and_ready(frontend, backend, "SELECT", nrows);
 
 	pfree(pools);
 }
@@ -1529,7 +1658,7 @@ void processes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backen
 		}
 	}
 
-	send_complete_and_ready(frontend, backend, nrows);
+	send_complete_and_ready(frontend, backend, "SELECT", nrows);
 
 	pfree(processes);
 }
@@ -1586,7 +1715,7 @@ void version_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 		pool_write(frontend, version[0].version, strlen(version[0].version));
 	}
 
-	send_complete_and_ready(frontend, backend, 1);
+	send_complete_and_ready(frontend, backend, "SELECT",1);
 
 	pfree(version);
 }
@@ -1708,7 +1837,7 @@ void cache_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 		}
 	}
 
-	send_complete_and_ready(frontend, backend, 1);
+	send_complete_and_ready(frontend, backend, "SELECT", 1);
 
 	pfree(strp);
 }
