@@ -5,7 +5,7 @@
 * pgpool: a language independent connection pool server for PostgreSQL
 * written by Tatsuo Ishii
 *
-* Copyright (c) 2003-2018	PgPool Global Development Group
+* Copyright (c) 2003-2016	PgPool Global Development Group
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
@@ -208,21 +208,30 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			cp->socket_state = POOL_SOCKET_ERROR;
 			if (cp->isbackend)
 			{
+				if (cp->con_info && cp->con_info->swallow_termination == 1)
+				{
+					cp->con_info->swallow_termination = 0;
+					ereport(FATAL,
+						(errmsg("unable to read data from DB node %d",cp->db_node_id),
+							 errdetail("pg_terminate_backend was called on the backend")));
+				}
+
 				/* if fail_over_on_backend_error is true, then trigger failover */
 				if (pool_config->fail_over_on_backend_error)
 				{
-					notice_backend_error(cp->db_node_id);
+					notice_backend_error(cp->db_node_id, true);
+
+                    /* If we are in the main process, we will not exit */
 					child_exit(POOL_EXIT_AND_RESTART);
-                    /* we are in main process */
-                    ereport(ERROR,
-						(errmsg("unable to read data from DB node %d",cp->db_node_id),
-                             errdetail("socket read failed with an error \"%s\"", strerror(errno))));
+					ereport(ERROR,
+							(errmsg("unable to read data from DB node %d",cp->db_node_id),
+							 errdetail("socket read failed with an error \"%s\"", strerror(errno))));
 				}
 				else
 				{
-                    ereport(ERROR,
-						(errmsg("unable to read data from DB node %d",cp->db_node_id),
-                             errdetail("socket read failed with an error \"%s\"", strerror(errno))));
+					ereport(ERROR,
+							(errmsg("unable to read data from DB node %d",cp->db_node_id),
+							 errdetail("socket read failed with an error \"%s\"", strerror(errno))));
 				}
 			}
 			else
@@ -245,12 +254,6 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
                 ereport(FATAL,
 					(errmsg("unable to read data from DB node %d",cp->db_node_id),
                          errdetail("EOF encountered with backend")));
-
-#ifdef NOT_USED
-			    /* fatal error, notice to parent and exit */
-			    notice_backend_error(IS_MASTER_NODE_ID(cp->db_node_id));
-				child_exit(POOL_EXIT_AND_RESTART);
-#endif
 			}
 			else
 			{
@@ -352,10 +355,18 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 			cp->socket_state = POOL_SOCKET_ERROR;
 			if (cp->isbackend)
 			{
+				if (cp->con_info && cp->con_info->swallow_termination == 1)
+				{
+					cp->con_info->swallow_termination = 0;
+					ereport(FATAL,
+						(errmsg("unable to read data from DB node %d",cp->db_node_id),
+							 errdetail("pg_terminate_backend was called on the backend")));
+				}
+
 				/* if fail_over_on_backend_error is true, then trigger failover */
 				if (pool_config->fail_over_on_backend_error)
 				{
-					notice_backend_error(cp->db_node_id);
+					notice_backend_error(cp->db_node_id, true);
 					child_exit(POOL_EXIT_AND_RESTART);
                     /* we are in main process */
                     ereport(ERROR,
@@ -384,12 +395,6 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
                 ereport(ERROR,
                     (errmsg("unable to read data from backend"),
                          errdetail("EOF read on socket")));
-
-#ifdef NOT_USED
-			    /* fatal error, notice to parent and exit */
-			    notice_backend_error(IS_MASTER_NODE_ID(cp->db_node_id));
-				child_exit(POOL_EXIT_AND_RESTART);
-#endif
 			}
 			else
 			{
@@ -519,7 +524,7 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 	wlen = len;
 
 	ereport(DEBUG1,
-			(errmsg("pool_write_flush: write size: %d", wlen)));
+			(errmsg("pool_write_flush_it: write size: %d", wlen)));
 
 	if (wlen == 0)
 	{
@@ -534,14 +539,14 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 
 		if (cp->ssl_active > 0)
 		{
-			sts = pool_ssl_write(cp, buf+offset, wlen);
+		  sts = pool_ssl_write(cp, buf, wlen);
 		}
 		else
 		{
-			sts = write(cp->fd, buf+offset, wlen);
+		  sts = write(cp->fd, buf, wlen);
 		}
 
-		if (sts >= 0)
+		if (sts > 0)
 		{
 			wlen -= sts;
 
@@ -554,7 +559,7 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 			else if (wlen < 0)
 			{
 				ereport(WARNING,
-						(errmsg("pool_write_flush: invalid write size %d", sts)));
+						(errmsg("pool_write_flush_it: invalid write size %d", sts)));
 				return -1;
 			}
 
@@ -562,7 +567,7 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 			{
 				/* need to write remaining data */
 				ereport(DEBUG1,
-						(errmsg("pool_write_flush: write retry: %d", wlen)));
+						(errmsg("pool_write_flush_it: write retry: %d", wlen)));
 
 				offset += sts;
 				continue;
@@ -619,49 +624,16 @@ int pool_flush_it(POOL_CONNECTION *cp)
 	{
 		errno = 0;
 
-#ifdef NOT_USED
-		if (!cp->isbackend)
+		if (cp->ssl_active > 0)
 		{
-			fd_set	writemask;
-			fd_set	exceptmask;
-
-			FD_ZERO(&writemask);
-			FD_ZERO(&exceptmask);
-			FD_SET(cp->fd, &writemask);
-			FD_SET(cp->fd, &exceptmask);
-
-			sts = select(cp->fd+1, NULL, &writemask, &exceptmask, NULL);
-			if (sts == -1)
-			{
-				if (errno == EAGAIN || errno == EINTR || errno == EWOULDBLOCK)
-					continue;
-				ereport(WARNING,
-						(errmsg("pool_flush_it: select() failed. reason: %s", strerror(errno))));
-				cp->wbufpo = 0;
-				return -1;
-			}
-			else if (sts == 0)
-			{
-				continue;
-			}
-			else if (FD_ISSET(cp->fd, &exceptmask))
-			{
-				ereport(LOG,
-					(errmsg("unable to flush data"),
-						 errdetail("exception occurred while in waiting select() ")));
-
-				cp->wbufpo = 0;
-				return -1;
-			}
-		}
-#endif
-		if (cp->ssl_active > 0) {
 		  sts = pool_ssl_write(cp, cp->wbuf + offset, wlen);
-		} else {
+		}
+		else
+		{
 		  sts = write(cp->fd, cp->wbuf + offset, wlen);
 		}
 
-		if (sts >= 0)
+		if (sts > 0)
 		{
 			wlen -= sts;
 
@@ -701,7 +673,7 @@ int pool_flush_it(POOL_CONNECTION *cp)
 			 * just report debug message.
 			 */
 			if (cp->isbackend)
-				ereport(DEBUG1,
+				ereport(WARNING,
 					(errmsg("write on backend %d failed with error :\"%s\"",cp->db_node_id,strerror(errno)),
 						 errdetail("while trying to write data from offset: %d wlen: %d",offset, wlen)));
 			else
@@ -727,10 +699,18 @@ int pool_flush(POOL_CONNECTION *cp)
 	{
 		if (cp->isbackend)
 		{
+			if (cp->con_info && cp->con_info->swallow_termination == 1)
+			{
+				cp->con_info->swallow_termination = 0;
+				ereport(FATAL,
+						(errmsg("unable to read data from DB node %d",cp->db_node_id),
+						 errdetail("pg_terminate_backend was called on the backend")));
+			}
+
 			/* if fail_over_on_backend_error is true, then trigger failover */
 			if (pool_config->fail_over_on_backend_error)
 			{
-				notice_backend_error(cp->db_node_id);
+				notice_backend_error(cp->db_node_id, true);
 				ereport(LOG,
 					(errmsg("unable to flush data to backend"),
 						 errdetail("do not failover because I am the main process")));
@@ -740,7 +720,7 @@ int pool_flush(POOL_CONNECTION *cp)
 			}
 			else
 			{
-                ereport(FATAL,
+                ereport(ERROR,
                     (errmsg("unable to flush data to backend"),
                          errdetail("do not failover because fail_over_on_backend_error is off")));
 			}
@@ -774,10 +754,18 @@ int pool_flush_noerror(POOL_CONNECTION *cp)
     {
         if (cp->isbackend)
         {
+			if (cp->con_info && cp->con_info->swallow_termination == 1)
+			{
+				cp->con_info->swallow_termination = 0;
+				ereport(FATAL,
+						(errmsg("unable to read data from DB node %d",cp->db_node_id),
+						 errdetail("pg_terminate_backend was called on the backend")));
+			}
+
             /* if fail_over_on_backend_erro is true, then trigger failover */
             if (pool_config->fail_over_on_backend_error)
             {
-                notice_backend_error(cp->db_node_id);
+                notice_backend_error(cp->db_node_id, true);
                 child_exit(POOL_EXIT_AND_RESTART);
 				ereport(LOG,
 					(errmsg("unable to flush data to backend"),
@@ -923,9 +911,12 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
             }
 		}
 
-		if (cp->ssl_active > 0) {
+		if (cp->ssl_active > 0)
+		{
 		  readlen = pool_ssl_read(cp, cp->sbuf+readp, readsize);
-		} else {
+		}
+		else
+		{
 		  readlen = read(cp->fd, cp->sbuf+readp, readsize);
 		}
 
@@ -934,7 +925,15 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 			cp->socket_state = POOL_SOCKET_ERROR;
 			if (cp->isbackend)
 			{
-				notice_backend_error(cp->db_node_id);
+				if (cp->con_info && cp->con_info->swallow_termination == 1)
+				{
+					cp->con_info->swallow_termination = 0;
+					ereport(FATAL,
+							(errmsg("unable to read data from DB node %d",cp->db_node_id),
+							 errdetail("pg_terminate_backend was called on the backend")));
+				}
+
+				notice_backend_error(cp->db_node_id, true);
 				child_exit(POOL_EXIT_AND_RESTART);
                 ereport(ERROR,
                         (errmsg("unable to read data from frontend"),
